pub fn ziv_retries_reset()Expand description
Reset the retry counter to 0 (before a measurement, so exact short-circuits that never enter a Ziv loop report 0 rather than the previous call’s count).
pub fn ziv_retries_reset()Reset the retry counter to 0 (before a measurement, so exact short-circuits that never enter a Ziv loop report 0 rather than the previous call’s count).