Module math

Module math 

Advanced mathematical functions

Modules

cache
Opt-in cache of mathematical constants, enabling progressive refinement.
hyper
Hyperbolic functions, built from the cancellation-free exp_m1 / ln_1p primitives:
trig
Trigonometric functions, built on top of the cached constants π/2 and the real exp/ln primitives:

Enums

FpError
Error returned by floating-point operations that cannot produce a usable result.

Type Aliases

FpResult
The result of a floating point operation: a correctly-rounded value (which may be an infinity produced as a value), or an FpError when the operation cannot proceed.